$CASE
$TITLE(Assembly example program)
;*==========================================================*
;*                                                          *
;*         SOURCE FILE : DEMO_ASM.ASM                       *
;*         PACKAGE     : I2C                                *
;*                                                          *
;* Hours and minutes will be displayed on LCD display       *
;* Dot between hours and minutes will blink                 *
;*                                                          *
;*==========================================================*
$DEBUG
$NOLIST


;*==========================================================*
;*    I N C L U D E S                                       *
;*==========================================================*
$INCLUDE(I2C_DATA.GLO)
$INCLUDE(I2C_MAC.DEF)
$LIST


;*==========================================================*
;*    G L O B A L   F U N C T I O N   D E F I N I T I O N S *
;*==========================================================*
    EXTRN CODE(_I2C_INIT)
	EXTRN CODE(_I2C_WRITE)
	EXTRN CODE(_I2C_READ_SUB)


;*==========================================================*
;*    L O C A L   D A T A   D E F I N I T I O N S           *
;*==========================================================*
RAMVAR      SEGMENT DATA    ;Segment for variables
STACK       SEGMENT DATA    ;Segment for variables
USER        SEGMENT CODE    ;Segment for application program


;*==========================================================*
;*      L O C A L   S Y M B O L   D E F I N I T I O N S     *
;*==========================================================*
CLOCK_ADR   EQU 0A2H        ;Address of PCF8583
CL_SUB_ADR  EQU 01H         ;Sub address for reading time
LCD_ADR     EQU 74H         ;Address of PCF8577


;*==========================================================*
;*    D A T A   S E G M E N T                               *
;*==========================================================*
        RSEG RAMVAR
TIME_BUFFER:    DS 4                  ;Buffer for I2C strings
LCD_BUFFER:     DS 5

        RSEG STACK
STACK_DATA:     DS 10


;*==========================================================*
;*    C O D E   S E G M E N T                               *
;*==========================================================*
        CSEG AT 00
        AJMP APPL_MAIN

        RSEG USER

APPL_MAIN:
        MOV  SP,#STACK_DATA-1
        MOV DPTR,#LCD_TAB           ;Pointer to segment table
        MOV LCD_BUFFER,#00          ;Ctrl word for LCD driver
        %I2C_INIT(22h,TIME_BUFFER,0)  ;Init I2C interface
        CLR A                       ;Prepare buffer 
        MOV TIME_BUFFER,A           ;for clock int.
        MOV TIME_BUFFER+1,A
        %I2C_WRITE(CLOCK_ADR,2,TIME_BUFFER) ;Initialise clock

REPEAT: %I2C_READ_SUB(CLOCK_ADR,4,TIME_BUFFER,CL_SUB_ADR)
                                    ;Read time


;*----------------------------------------------------------*
;*      Time has been read. Order:                          *
;*               hundreds of sec's, sec's, min's and hr's   *
;*----------------------------------------------------------*
        MOV A,TIME_BUFFER+3             ;Mask of hour counter
        ANL A,#3Fh
        MOV TIME_BUFFER+3,A

        ACALL CONVERT                   ;Convert time data to
                                        ;LCD segment data

;*----------------------------------------------------------*
;*		Check if dot has to be switched on					*
;*----------------------------------------------------------*
        ORL LCD_BUFFER+3,#01h


;*----------------------------------------------------------*
;*		If lsb of seconds is '0', then switch on dp         *
;*----------------------------------------------------------*
        MOV A,TIME_BUFFER+1             ;Get seconds
        RRC A
        JC PROCEED                     
        ORL LCD_BUFFER+1,#01            ;Switch on dp


;*----------------------------------------------------------*
;*		Display new time									*
;*----------------------------------------------------------*
PROCEED: %I2C_WRITE(LCD_ADR,5,LCD_BUFFER)
        SJMP REPEAT                     ;Read new time


;*----------------------------------------------------------*
;*		CONVERT converts BCD data of time to segment data	*
;*----------------------------------------------------------*
CONVERT:MOV R0,#LCD_BUFFER+1			;R0 is pointer
        MOV A,TIME_BUFFER+3             ;Get hours
        SWAP A                          ;Swap nibbles
        ACALL LCD_DATA                  ;Convrt 10's of hours
        MOV A,TIME_BUFFER+3
        ACALL LCD_DATA                  ;Convert hours
        MOV A,TIME_BUFFER+2             ;Get minutes
        SWAP A
        ACALL LCD_DATA                  ;Convrt 10's of minut
        MOV A,TIME_BUFFER+2
        ACALL LCD_DATA                  ;Convert minutes
        RET


;*----------------------------------------------------------*
;*		LCD_DATA gets data from segment table and			*
;*		stores it in LCD_BUFFER 							*
;*----------------------------------------------------------*
LCD_DATA:
        ANL A,#0FH                      ;Mask off LS-nibble
        MOVC A,@A+DPTR                  ;Get segment data
        MOV @R0,A                       ;Save segment data
        INC R0
        RET



;*----------------------------------------------------------*
;*		Conversion table for LCD							*
;*----------------------------------------------------------*
LCD_TAB:
        DB 0FCH,60H,0DAH                ;'0','1','2'
        DB 0F2H,66H,0B6H                ;'3','4','5'
        DB 3EH,0E0H,0FEH                ;'6','7','8'
        DB 0E6H                         ;'9'
;
        END
