$TITLE(I2C_SLAVE)
;*==========================================================*
;*                                                          *
;*         SOURCE FILE : I2C_SLAV.ASM                       *
;*         PACKAGE     : I2C                                *
;*                                                          *
;*==========================================================*
$DEBUG


;*==========================================================*
;*    I N C L U D E S                                       *
;*==========================================================*
$NOLIST
$INCLUDE(I2C_DATA.GLO)
$INCLUDE(REG751.H)
$LIST


;*==========================================================*
;*    G L O B A L   R E F E R E N C E S                     *
;*==========================================================*
    EXTRN CODE(I2C_TRX_BYTE)
    EXTRN CODE(I2C_RCV_BYTE)
    EXTRN CODE(I2C_RCV_ADDR)
    EXTRN CODE(ADDR_COMPARE)


;*==========================================================*
;*    G L O B A L   F U N C T I O N   D E F I N I T I O N S *
;*==========================================================*
    PUBLIC I2C_SLV_TRX
    PUBLIC I2C_SLV_RCV
    PUBLIC ADDR_RECOG

;*==========================================================*
;*    L O C A L   S Y M B O L   D E C L A R A T I O N S     *
;*==========================================================*
    SLV_BUF_PTR_W   SET         R1
    BIT_CNT         SET         R3
    I2C_RELEASE     EQU         0F4H


;*MPF:::I2C_SLAVE===========================================*
;*                                                          *
;* FUNCTION NAME:      I2C_SLAVE_ROUTINES                   *
;* DESCRIPTION:        I2C                                  *
;* DESCRIPTION:                                             *
;*    This file contains an example of how to make a slave  *
;*    transmitter and slave receiver function. The slave    *
;*    transmitter functions tranmits byte from a buffer,    *
;*    while the slave receiver routine  receives bytes into *
;*    a buffer. The buffer pointer is loaded during the     *
;*    I2C_INIT routine.                                     *
;*                                                          *
;*EMP=======================================================*


;*==========================================================*
;*    I N T E R R U P T   C O D E   S E G M :   I I C       *
;*==========================================================*
    CSEG AT 023H

        PUSH    ACC
        PUSH    PSW
        MOV     PSW,#8
        AJMP    ADDR_RECOG


;*==========================================================*
;*    C O D E   S E G M E N T                               *
;*==========================================================*
    I2C_DRIVER SEGMENT CODE
    RSEG I2C_DRIVER


;*MPF:::I2C::I2C_SLAV.ASM:I2C_ADDR_RECOG====================*
;*                                                          *
;* FUNCTION NAME:      I2C_ADDR_RECOG                       *
;* PACKAGE NAME:       I2C                                  *
;* DESCRIPTION:                                             *
;*    If an I2C interrupt occurs, and the STR bit is set,   *
;*    I2C_ADDR_RECOG  receives the incoming slave address.  *
;*    If it's own slave address is recognized, the slave    *
;*    receiver or slave transmitter routine (depending on   *
;*    the R/WN bit) is called                               *
;*                                                          *
;* INPUT:  --                                               *
;* OUTPUT: I2C_ERROR byte  (bit addressable)                *
;*                                                          *
;*EMP=======================================================*
ADDR_RECOG:
        MOV     I2CON,#C_STRT
        ACALL   I2C_RCV_ADDR
        JB      I2C_ERR,EXIT_SLV_AD
        ACALL   ADDR_COMPARE
EXIT_SLV_AD:
        CLR     I2C_ERR
        MOV     I2CON,#I2C_RELEASE
        POP     PSW
        POP     ACC
        RETI


;*MPF:::I2C::I2C_SLAV.ASM:I2C_SLV_TRX=======================*
;*                                                          *
;* FUNCTION NAME:      I2C_SLV_TRX                          *
;* PACKAGE NAME:       I2C                                  *
;* DESCRIPTION:                                             *
;*    After the SLV_ADDR/W is received, the I2C_SLV_TRX     *
;*    transmits a byte from the slave buffer. The pointer   *
;*    to this buffer is loaded during the I2C_INIT function.*
;*    If an acknowledge is received, the pointer is         *
;*    incremented and the next byte is transmitted. The     *
;*    function is exit on reception of a NACK.              *
;*                                                          *
;*    Normally the slave routines are entered through an I2C*
;*    interrupt, but if the 8x751 loses abitration during   *
;*    the slave address and it recognises it's own slave    *
;*    address/W, the I2C_SLV_TRX function is entered at XXXX*
;*                                                          *
;* PROTOCOL: <S><SLV_ADDR><W><D0><A><D1><A>...<A><Dn><N><P> *
;*                                                          *
;* REGISTER USAGE : Register bank 1, is used during the I2C *
;*        routines, it contains no static data, and is free *
;*        for the user outside the I2C routines             *
;*                                                          *
;* INPUT:  --                                               *
;* OUTPUT: I2C_ERROR byte  (bit addressable)                *
;*                                                          *
;*EMP=======================================================*
I2C_SLV_TRX:
        MOV     SLV_BUF_PTR_W,SLV_BUF_PTR
S_TRX:
        MOV     A,@SLV_BUF_PTR_W
        ACALL   I2C_TRX_BYTE
        JB      I2C_ERR,EXIT_SLV_TRX
        JB      NO_ACK,EXIT_SLV_TRX
        INC     SLV_BUF_PTR_W
        AJMP    S_TRX
EXIT_SLV_TRX:
        RET


;*MPF:::I2C::I2C_SLAV.ASM:I2C_SLV_RCV=======================*
;*                                                          *
;* FUNCTION NAME:      I2C_SLV_RCV                          *
;* PACKAGE NAME:       I2C                                  *
;* DESCRIPTION:                                             *
;*    After the SLV_ADDR/R is received, the I2C_SLV_RCV     *
;*    receives a byte into the slave buffer. The pointer    *
;*    to this buffer is loaded during the I2C_INIT function.*
;*    After the byte is received, an acknowledge is send,   *
;*    the pointer is incremented and the next byte is       *
;*    received. The function is exit on if a start condition*
;*    is detected                                           *
;*                                                          *
;*    Normally the slave routines are entered through an I2C*
;*    interrupt, but if the 8x751 loses abitration during   *
;*    the slave address and it recognises it's own slave    *
;*    address/R, the I2C_SLV_RCV function is entered at xx  *
;*                                                          *
;* PROTOCOL: <S><SLV_ADDR><R><D0><A><D1><A>...<A><Dn><A><P> *
;*                                                          *
;* REGISTER USAGE : Register bank 1, is used during the I2C *
;*        routines, it contains no static data, and is free *
;*        for the user outside the I2C routines             *
;*                                                          *
;* INPUT:  --                                               *
;* OUTPUT: I2C_ERROR byte  (bit addressable)                *
;*                                                          *
;*EMP=======================================================*
I2C_SLV_RCV:
        MOV     SLV_BUF_PTR_W,SLV_BUF_PTR
S_RCV:
        ACALL   I2C_RCV_BYTE
        JB      I2C_ERR,EXIT_SLV_RCV
        MOV     @SLV_BUF_PTR_W,A        ;save byte
        MOV     I2DAT,#0                ;send ACK
        JNB     ATN,$
        JNB     DRDY,EXIT_SLV_RCV
        INC     SLV_BUF_PTR_W
        AJMP    S_RCV
EXIT_SLV_RCV:
        RET



;*==========================================================*
;*     H I S T O R Y                                        *
;*==========================================================*
;*                                                          *
;* 21-05-91    J.C. Pijnenburg  original version            *
;*                                                          *
;*==========================================================*
        END
