$TITLE(I2C_Write_Sub_SWinc command)
;*==========================================================*
;*                                                          *
;*         SOURCE FILE : I2C_WSWI.ASM                       *
;*         PACKAGE     : I2C                                *
;*                                                          *
;*==========================================================*
$DEBUG


;*==========================================================*
;*    I N C L U D E S                                       *
;*==========================================================*
$NOLIST
$INCLUDE(I2C_DATA.GLO)
$INCLUDE(I2C_DATA.LOC)
$INCLUDE(REG751.H)
$LIST


;*==========================================================*
;*    G L O B A L   R E F E R E N C E S                     *
;*==========================================================*
    EXTRN CODE(I2C_MESS_HAND)


;*==========================================================*
;*    G L O B A L   F U N C T I O N   D E F I N I T I O N S *
;*==========================================================*
    PUBLIC _I2C_WRITE_SUB_SWINC
    PUBLIC _I2C_WRITE_MEMORY


;*==========================================================*
;*    L O C A L   S Y M B O L   D E C L A R A T I O N S     *
;*==========================================================*
    WRITE_SUB_SWINC_MASK  EQU 00CH
    WRITE_MEMORY_MASK     EQU 02CH
               ;REP_STRT_BLK1  = 0    (NO)
               ;RWN_BLK1       = 0    (WRITE)
               ;ADDR2          = 1    (YES)
               ;ADDR2_SUB      = 1    (YES)
               ;BLOCK2         = 0    (NO)
               ;RWN_BLK2       = 0 or 1 (no blk2,
               ;                   used for delay/no delay)
               ;REP_STRT_BLK2  = 0    (--)
               ;TEST_DEVICE    = 0    (--)


;*==========================================================*
;*    C O D E   S E G M E N T                               *
;*==========================================================*
    I2C_DRIVER SEGMENT CODE
    RSEG I2C_DRIVER


;*MPF:::I2C::I2C_WSWI.ASM:I2C_WRITE_SUB_SWINC===============*
;*                                                          *
;* FUNCTION NAME:      I2C_WRITE_SUB_SWINC                  *
;* PACKAGE:            I2C                                  *
;* DESCRIPTION:                                             *
;*   Transmit an I2C message, the message is split into     *
;*   sub messages. Each sub message transmits one byte.     *
;*   If the slave is an EEPROM, a delay is generated after  *
;*   each sub message. The RWN_BLK2 is not used in the      *
;*   message handler (no block 2) and is therefor free to   *
;*   distinguish between write to EEPROM (1=delay) and other*
;*   (0= no delay)                                          *
;*                                                          *
;* PROTOCOL:                                                *
;*    <S><SLV_ADDR><W><A><SUB_ADDR><A><D0><A><P>            *
;*         if (RWN_BLK2) delay 40 ms                        *
;*    <S><SLV_ADDR><W><A><SUB_ADDR+1><A><D1><P>             *
;*         if (RWN_BLK2) delay 40 ms                        *
;*    <-    --      -  - --   ->                            *
;*         if (RWN_BLK2) delay 40 ms                        *
;*    <S><SLV_ADDR><W><A><SUB_ADDR+n-1><A><Dn-1><P>         *
;*         if (RWN_BLK2) delay 40 ms                        *
;*                                                          *
;* INPUT: Message control byte I2C_CTRL (bit addressable)   *
;*        Message control block I2C_MCB, containing:        *
;*          I2C_ADDR1       (slave address)                 *
;*          BUF_LEN1        (number of bytes (mess) to trx.)*
;*          BUF_PTR1        (ptr to transmit buffer)        *
;*          I2C_ADDR2       (sub address)                   *
;*                                                          *
;* OUTPUT: I2C_ERROR byte  (bit addressable)                *
;*                                                          *
;*EMP=======================================================*

_I2C_WRITE_MEMORY:
        MOV     I2C_CTRL,#WRITE_MEMORY_MASK
        SJMP    SET_MESS_CNT
_I2C_WRITE_SUB_SWINC:
        MOV     I2C_CTRL,#WRITE_SUB_SWINC_MASK

SET_MESS_CNT:
        MOV     MEM_MESS_LEN,I2C_MCB+1
        MOV     I2C_MCB+1,#1            ;set BUF_LEN_1 = 1
SUB_MESS:
        ACALL   I2C_MESS_HAND
        JB      I2C_ERR,END_WSWI
        INC     I2C_MCB+2               ;inc. BUF_PTR_1
        INC     I2C_MCB+3               ;inc. SUB_ADDR
        JNB     RWN_BLK2,NEXT
        MOV     MEM_DELAY_H,#EEPROM_PROG_DELAY
        MOV     MEM_DELAY_L,#00         ; 40 mS delay at 16MHz
PROGRAM_DELAY:
        DJNZ    MEM_DELAY_L,$
        DJNZ    MEM_DELAY_H,PROGRAM_DELAY
NEXT:
        DJNZ    MEM_MESS_LEN,SUB_MESS
END_WSWI:
        RET

        END
