/*************************************************
 Na podstawie AN187 firmy MAXIM-DALLAS
 2004/04 Jacek Bogusz
 
 kompilator RC-51
 *************************************************/

#pragma SMALL 
#include <reg51.h>
#include <stdio.h>

//************* definicje staych dla preprocesora ********************
#define FALSE 0
#define TRUE 1
#define byte unsigned char

#define	XTAL	110592					//definicja rezonatora (8..25MHz!)
#define	nop()	ACC++						//opnienie, 1 cykl maszynowy

sbit	one_wire_IO = P1^0;				//definicja podczenia linii portu 1W

//****************** deklaracje metod i funkcji ***********************

//wyszukanie pierwszego ukadu na magistrali 1-Wire
int one_wire_FIRST();
//wyszukanie nastpnego ukadu
int one_wire_NEXT();
//weryfikacja,czy ukad jest doczony do magistrali
int one_wire_VERIFY();
//wyszukiwanie ukadw w konkretnej grupie
void one_wire_TSETUP(byte family_code);
//omijanie grupy ukadw
void one_wire_FAMILYSKIP();
//wysanie sygnau reset na magistral 1-Wire
int one_wire_RESET();
//wysanie bajtu na magistral 1-Wire
void one_wire_WRBYTE(byte byte_value);
//wysanie bitu na magistral 1-Wire
void one_wire_WRBIT(bit bit_value);
//odczyt bitu z magistrali 1-Wire
byte one_wire_RDBIT();
//oglna funkcja wyszukiwania
int one_wire_SEARCH();
//wyliczenie sumy kontrolnej odebranych bajtw
byte docrc8(byte value);

//************************ zmienne globalne ***************************
//adres odnalezionego ukadu
byte ROM_NO[8];
//ostatnio odnaleziona sprzeczno
int LastDiscrepancy;
//ostatnio odnaleziona sprzeczno w obrbie 8-bitw rodziny ukadw
int LastFamilyDiscrepancy;
//flaga sygnalizujca, e ukad jest ostatnim z danej grupy lub doczonych ukadw
int LastDeviceFlag;
//suma kontrolna CRC8
byte crc8;


//--------------------------------------------------------------------------
// Odnajduje "pierwszy" ukad na magistrali 1-Wire
// Zwraca TRUE, jeli odnaleziono ukad (adres zawarty w ROM_NO)
// Zwraca FALSE, jeli nie ma ukadu
int one_wire_FIRST()
{
//ustalenie warunkw pocztkowych
	LastDiscrepancy = LastFamilyDiscrepancy = 0;
	LastDeviceFlag = FALSE;
	return (one_wire_SEARCH());
}


//--------------------------------------------------------------------------
// Odnajduje "nastpny" ukad na magistrali 1-Wire
// Zwraca TRUE, jeli odnaleziono ukad (adres zawarty w ROM_NO)
// Zwraca FALSE, jeli nie ma ukadu
int one_wire_NEXT()
{
//ustalenie war.pocztkowych polega na nie zmienianiu poprzedniego stanu
	return one_wire_SEARCH();
}


//--------------------------------------------------------------------------
//Oglna funkcja wyszukiwania; podstawowa implementacja algorytmu
// Zwraca TRUE, jeli odnaleziono ukad (adres zawarty w ROM_NO)
// Zwraca FALSE, jeli nie ma ukadu; koniec przeszukiwania
int one_wire_SEARCH()
{
	int id_bit_number;
	int last_zero, rom_byte_number, search_result;
	int id_bit, cmp_id_bit;
	byte rom_byte_mask, search_direction;
//inicjalizacja zmiennych
	id_bit_number = 1;
	last_zero = 0;
	rom_byte_number = 0;
	rom_byte_mask = 1;
	search_result = 0;
	crc8 = 0;
//jeli poprzednie wywoania nie odszukao ostatniego ukadu
	if (!LastDeviceFlag)
	{
//1-Wire reset
		if (!one_wire_RESET())
		{
//inizjalizacja zmiennych
			LastDiscrepancy = LastFamilyDiscrepancy = 0;
			LastDeviceFlag = FALSE;
			return FALSE;
		}
//wysanie komendy "szukaj"
		one_wire_WRBYTE(0xF0);
//ptla,w ktrej przeprowadzane jest szukanie
		do
		{
//odczytaj bit i bit komplementarny
			id_bit = one_wire_RDBIT();
			cmp_id_bit = one_wire_RDBIT();
//sprawd warunek rnych wartoci bitu i bitu komplementarnego
			if ((id_bit == 1) && (cmp_id_bit == 1)) break; else
			{
//bit adresu 0 lub 1 dla wszystkich doczonych ukadw
				if (id_bit != cmp_id_bit) search_direction = id_bit; // bit write value for search
				else
				{
					if (id_bit_number < LastDiscrepancy) 
						search_direction = ((ROM_NO[rom_byte_number] & rom_byte_mask) > 0);
					else search_direction = (id_bit_number == LastDiscrepancy);
					if (search_direction == 0)
					{
						last_zero = id_bit_number;
//sprawd, czy to ostatni warunek sprzeczny w obrbie rodziny ukdw
						if (last_zero < 9)
						LastFamilyDiscrepancy = last_zero;
					}
				}
//ustaw lub wyzeruj bit w zmiennej rom_byte_number
//za pomoc maski rom_byte_mask
				if (search_direction == 1) 
					ROM_NO[rom_byte_number] |= rom_byte_mask; else ROM_NO[rom_byte_number] &= ~rom_byte_mask;
//wylij bit kierunku poszukiwania
				one_wire_WRBIT(search_direction & 0x01);
//zwiksz licznik id_bit_number i przesu w lewo mask rom_byte_mask
				id_bit_number++;
				rom_byte_mask <<= 1;
//jeli maska jest 0, to we nowy adres rom_byte_number kasuj mask
				if (rom_byte_mask == 0)
				{
//wylicz CRC8
					docrc8(ROM_NO[rom_byte_number]);
					rom_byte_number++;
					rom_byte_mask = 1;
				}
			}
//wykonaj dla wszystkich bajtw adresu
		}while(rom_byte_number < 8);

//jeli poszukiwanie zakoczyo si sukcesem
		if (!((id_bit_number < 65) || (crc8 != 0)))
		{
//ustaw zmienne LastDiscrepancy, LastDeviceFlag, search_result
			LastDiscrepancy = last_zero;
//sprawd, czy to ostatni ukad
			if (LastDiscrepancy == 0)
			LastDeviceFlag = TRUE;
			search_result = TRUE;
		}
	}
//jeli nie znaleziono adnego ukadu
	if (!search_result || !ROM_NO[0])
	{
		LastDiscrepancy = 0;
		LastDeviceFlag = FALSE;
		LastFamilyDiscrepancy = 0;
		search_result = FALSE;
	}
	return search_result;
}


//--------------------------------------------------------------------------
// Weryfikuje,czy urzdzenie od podanym adresie jest doczone do 1-Wire
// Zwraca TRUE, jeli tak
int one_wire_VERIFY()
{
	byte rom_backup[8];
	int i,rslt,ld_backup,ldf_backup,lfd_backup;
//przechowaj kopi stanu aktualnego
	for (i = 0; i < 8; i++) rom_backup[i] = ROM_NO[i];
	ld_backup = LastDiscrepancy;
	ldf_backup = LastDeviceFlag;
	lfd_backup = LastFamilyDiscrepancy;
//ustaw procedur wyszukiwania
	LastDiscrepancy = 64;
	LastDeviceFlag = FALSE;
	if (one_wire_SEARCH())
	{
//sprawd czy znaleziono ten sam ukad
		rslt = TRUE;
		for (i = 0; i < 8; i++)
		{
			if (rom_backup[i] != ROM_NO[i])
			{
				rslt = FALSE;
				break;
			}
		}
	} else rslt = FALSE;
//odtworzenie stanu pocztkowego
	for (i = 0; i < 8; i++) ROM_NO[i] = rom_backup[i];
	LastDiscrepancy = ld_backup;
	LastDeviceFlag = ldf_backup;
	LastFamilyDiscrepancy = lfd_backup;
//zwr rezultat weryfikacji
	return rslt;
}


//--------------------------------------------------------------------------
// Ustawienie warunkw dla wyszukiwania ukadw w rodzinie, po tym mona
// uruchomi funkcj "szukaj nastpnego" (one_wire_NEXT)
void one_wire_TSETUP(byte family_code)
{
	int i;
	ROM_NO[0] = family_code;
	for (i = 1; i < 8; i++) ROM_NO[i] = 0;
	LastDiscrepancy = 64;
	LastFamilyDiscrepancy = 0;
	LastDeviceFlag = FALSE;
}


//--------------------------------------------------------------------------
// Ustawienie warunkw do pomijania przy wyszukiwaniu danej rodziny ukadw
// dla funkcji one_wire_NEXT().
void one_wire_FAMILYSKIP()
{
	LastDiscrepancy = LastFamilyDiscrepancy;
	LastFamilyDiscrepancy = 0;
	if (LastDiscrepancy == 0) LastDeviceFlag = TRUE;
}


//--------------------------------------------------------------------------
// Wysanie sygnau reset i sprawdzenie obecnoci ukadw na magistrali;
// zwracane jest TRUE, jeli ukady s obecne
int one_wire_RESET()
{
	byte delay;
	bit err;
//ptla opnienia 480 < t < 960 cykli
	delay = (byte)(XTAL/12e6*480/4);
//ptle opniajce su do wytworzenia tzw. time slots, ktrych dokadny opis
//mona znale w opisie standardu 1-Wire
	do					
	{					
		one_wire_IO = 0;		
		nop();
	}while(--delay);
//ptla opnienia 60 < t < 75 cm.
	delay = (byte)( XTAL / 12e6 * 66 / 2 );
	do 
	{
		one_wire_IO = 1;
	}while(--delay);
//stan niski oznacza, e urzdzenie(a) 1W jest(s) podczone
	err = one_wire_IO;			
//opnienie 480 < t
	delay = (byte)(XTAL/12e6*(480-66)/4);
	do												{
		nop(); 
		nop();
	}while(--delay);

	err = one_wire_IO;
//stan niski linii portu 1W oznacza bd 
	return (!err);
}


//--------------------------------------------------------------------------
// Przesanie lub odczyt bitu przez lini 1W
bit one_wire_bit_io(bit b)
{
	byte delay;

	delay = (byte)(XTAL/12e6*15/2-2);		//15 > t
	one_wire_IO = 0;											//1
	one_wire_IO = b;											//3
	while(--delay);											//3 + delay * 2
	b = one_wire_IO;
	delay = (byte)(XTAL/12e6*45/2);			//60 < t
	while(--delay);
	one_wire_IO = 1;
	return b;
}


//--------------------------------------------------------------------------
// Przesanie bajtu przez magistral 1-Wire
//
void one_wire_WRBYTE(byte byte_value)
{
	byte bit_counter = 8;
	do 
	{
		byte_value = byte_value >> 1 | (one_wire_bit_io(byte_value & 0x01) ? 0x80 : 0);
	}while(--bit_counter);
}


//--------------------------------------------------------------------------
// Przesanie pojedynczego bitu przez magistral 1-Wire
void one_wire_WRBIT(bit bit_value)
{
	one_wire_bit_io(bit_value);
}


//--------------------------------------------------------------------------
// Odczyt pojedynczego bitu z magistrali 1-Wire
byte one_wire_RDBIT()
{
	return (one_wire_bit_io(1));
}


//--------------------------------------------------------------------------
// Tablica wartoci do wyznaczenia sumy kontrolnej CRC8
code byte dscrc_table[] = {
0, 94,188,226, 97, 63,221,131,194,156,126, 32,163,253, 31, 65,
157,195, 33,127,252,162, 64, 30, 95, 1,227,189, 62, 96,130,220,
35,125,159,193, 66, 28,254,160,225,191, 93, 3,128,222, 60, 98,
190,224, 2, 92,223,129, 99, 61,124, 34,192,158, 29, 67,161,255,
70, 24,250,164, 39,121,155,197,132,218, 56,102,229,187, 89, 7,
219,133,103, 57,186,228, 6, 88, 25, 71,165,251,120, 38,196,154,
101, 59,217,135, 4, 90,184,230,167,249, 27, 69,198,152,122, 36,
248,166, 68, 26,153,199, 37,123, 58,100,134,216, 91, 5,231,185,
140,210, 48,110,237,179, 81, 15, 78, 16,242,172, 47,113,147,205,
17, 79,173,243,112, 46,204,146,211,141,111, 49,178,236, 14, 80,
175,241, 19, 77,206,144,114, 44,109, 51,209,143, 12, 82,176,238,
50,108,142,208, 83, 13,239,177,240,174, 76, 18,145,207, 45,115,
202,148,118, 40,171,245, 23, 73, 8, 86,180,234,105, 55,213,139,
87, 9,235,181, 54,104,138,212,149,203, 41,119,244,170, 72, 22,
233,183, 85, 11,136,214, 52,106, 43,117,151,201, 74, 20,246,168,
116, 42,200,150, 21, 75,169,247,182,232, 10, 84,215,137,107, 53};


//--------------------------------------------------------------------------
// Wyliczenie sumy kontrolnej CRC8
byte docrc8(byte value)
{
	crc8 = dscrc_table[crc8 ^ value];
	return crc8;
}


//--------------------------------------------------------------------------
// Program gwny, demonstracja sposobu uycia funkcji
void main()
{
	int rslt,i,cnt;
//wyszukiwanie wszystkich ukadw 1-Wire
	printf("\nSZUKAM WSZYSTKICH\n");
	cnt = 0;
	rslt = one_wire_FIRST();
	while (rslt)
	{
//wypisanie odnalezionego adresu ukadu
		for (i = 7; i >= 0; i--)
		printf("%02X", ROM_NO[i]);
		printf(" %d\n",++cnt);
		rslt = one_wire_NEXT();
	}

//odnalezienie ukadw wycznie z rodziny 0x1A
	printf("\nSZUKAM RODZINY '0x1A'\n");
	cnt = 0;
	one_wire_TSETUP(0x1A);
	while (one_wire_NEXT())
	{
//weryfikacja odnalezionych ukadw
		if (ROM_NO[0] != 0x1A)
		break;
//wypisanie odnalezionych adresw ukadw
		for (i = 7; i >= 0; i--)
		printf("%02X", ROM_NO[i]);
		printf(" %d\n",++cnt);
	}
	
//odnalezienie wszystkich za wyjtkiem rodzin o kodach 0x04, 0x1A, 0x23, and 0x01
	printf("\nSZUKAM WSZYSTKICH Z WYJATKIEM: 0x10, 0x04, 0x0A, 0x1A, 0x23, 0x01\n");
	cnt = 0;
	rslt = one_wire_FIRST();
	while (rslt)
	{
//weryfikacja
		if ((ROM_NO[0] == 0x04) || (ROM_NO[0] == 0x1A) ||
		(ROM_NO[0] == 0x01) || (ROM_NO[0] == 0x23) ||
		(ROM_NO[0] == 0x0A) || (ROM_NO[0] == 0x10))
		one_wire_FAMILYSKIP();
		else
		{
//wypisanie
			for (i = 7; i >= 0; i--)
			printf("%02X", ROM_NO[i]);
			printf(" %d\n",++cnt);
		}
		rslt = one_wire_NEXT();
	}
}

