/******************************************************************
  Program do obsugi funkcji interfejsu sprztowego I2C PCF8584
  Na podstawie AN95068 firmy Philips

  Raisonance RC-51
 ******************************************************************/
#include "stdio.h"
#include "i2cexprt.h"
#include "i2cdrivr.h"
#include "reg8584.h"

static unsigned char drvStatus;		//status zwracany przez driver

static I2C_MESSAGE	*p_iicMsg[2];	//wskanik do obaszaru komunikatw I2C
static I2C_TRANSFER	iicTfr;


/* na wejciu: status to stan drivera po zakoczeniu operacji, msgsDone to liczba przesanych komunikatw
   funkcja nie zwraca adnych wartoci, przekazywana jako parametr i wywoywana przez funkcj obsugi statusu
   drivera */
static void I2cReady(unsigned char status, unsigned char msgsDone)
{
	drvStatus = status;
}


/* funkcja uruchamia transfer I2C i oczekuje (ustawiajc limit czasu operacji) a transmisja danych 
	zostanie zakoczona przy uruchomieniu nie s wymagane adane parametry, po zakoczeniu pracy pole 
	statusu I2C_TRANSFER zawiera jedn z wartoci:
   	I2C_OK	transmisja przebiega pomylnie,
   	I2C_TIME_OUT	przekroczono limit czasu operacji,
   	inne kody bdw */
static void StartTransfer(void)
{
	long timeOut;								//licznik limitu czasu operacji
	unsigned char retries = 0;				//licznik limitu iloci powtrze
	
	do
	{
		drvStatus = I2C_BUSY;
		I2C_Transfer(&iicTfr, I2cReady);
		
		timeOut = 0;
		while (drvStatus == I2C_BUSY)		//oczekiwanie na zwolnienie magistrali I2C
		{
			if (++timeOut > 60000)
				drvStatus = I2C_TIME_OUT;	//przekroczono limit czasu operacji
			if (intMask == 0)					// 0 -> odpytywanie
			{
				if (!(CR_8584 & PIN_MASK))
					MainStateHandler();		//oczekiwanie a bit PIN przyjmie warto 0
			}
		}
		
		if (retries >= 6)						//sprawdzenie warunku do iloci powtrze operacji
		{
			printf("przekroczono dopuszczalny czas operacji\n");
		}
		else
			retries++;
		
		switch (drvStatus)					//wysanie komunikatu zalenego od statusu przeprowadzonej operacji
		{
			case I2C_OK:
				break;
			case I2C_NO_DATA:
				printf("bufor danych pusty\n");
				break;
			case I2C_NACK_ON_DATA:
				printf("brak potwierdzenia odbioru bloku danych\n");
				break;
			case I2C_NACK_ON_ADDRESS:
				printf("brak potwierdzenia odbioru bloku adresu\n");
				break;
			case I2C_DEVICE_NOT_PRESENT:
				printf("urzadzenie nie jest obecne\n");
				break;
			case I2C_ARBITRATION_LOST:
				printf("przegrana w procedurze arbitrazu\n");
				break;
			case I2C_TIME_OUT:
				printf("przekroczono limit czasu operacji\n");
				break;
			default:
				printf("status nieznany\n");
				break;
		}
	} while (drvStatus != I2C_OK);
}


/* jako parametr procedury podawany jest wskanik do komunikatu I2C przesyanego do urzdzenia slave
	protok: <Start><SlvA><Write><Ack><D1><Ack>...<Dnum><N><Stop> */
void I2C_Write(I2C_MESSAGE *msg)
{
	iicTfr.nrMessages = 1;
	iicTfr.p_message = p_iicMsg;
	p_iicMsg[0] = msg;
	
	StartTransfer();
}


/* zapisuje dwa komunikaty do rnych urzdze slave jako separatora uywajc powtrzonego warunku start
   protok: <Start><Slv1A><Write><Ack><D1><Ack>...<Dnum1><Ack>
             <Start><Slv2A><Write><Ack><D2><Ack>...<Dnum2><Ack><Stop> */
void I2C_WriteRepWrite(I2C_MESSAGE *msg1, I2C_MESSAGE *msg2)
{
	iicTfr.nrMessages = 2;
	iicTfr.p_message = p_iicMsg;
	p_iicMsg[0] = msg1;
	p_iicMsg[1] = msg2;
	
	StartTransfer();
}


/* wysya komunikat i odczytuje komunikat z dwch rnych urzdze slave uywajc jako separatora powtrzonego
   warunku start; oczywicie operacja moe dotyczy rwnie tego samego urzdzenia slave
   protok: <Start><Slv1A><Write><Ack><D1><Ack>...<Dnum1><Ack>
             <Start><Slv2A><Read><Ack><D2><Ack>...<Dnum2><Nack><Stop> */
void I2C_WriteRepRead(I2C_MESSAGE *msg1, I2C_MESSAGE *msg2)
{
	iicTfr.nrMessages = 2;
	iicTfr.p_message = p_iicMsg;
	p_iicMsg[0] = msg1;
	p_iicMsg[1] = msg2;
	
	StartTransfer();
}


/* odczytuje komunikat z urzdzenia slave
   protok: <Start><SlvA><Read><Ack><D1><Ack>...<Dnum1><Nack><Stop> */
void I2C_Read(I2C_MESSAGE *msg)
{
	iicTfr.nrMessages = 1;
	iicTfr.p_message = p_iicMsg;
	p_iicMsg[0] = msg;
	
	StartTransfer();
}


/* odczytuje komunikat z dwch rnych urzdze slave uywajc jako separatora powtrzonego
   warunku start; oczywicie operacja moe dotyczy rwnie tego samego urzdzenia slave
   protok: <Start><Slv1A><Read><Ack><D1><Ack>...<Dnum1><Nack>
             <Start><Slv2A><Read><Ack><D2><Ack>...<Dnum2><Nack><Stop> */
void I2C_ReadRepRead(I2C_MESSAGE *msg1, I2C_MESSAGE *msg2)
{
	iicTfr.nrMessages = 2;
	iicTfr.p_message = p_iicMsg;
	p_iicMsg[0] = msg1;
	p_iicMsg[1] = msg2;
	
	StartTransfer();
}


/* odczytuje i wysya komunikat komunikat do dwch rnych urzdze slave uywajc jako separatora powtrzonego
   warunku start; oczywicie operacja moe dotyczy rwnie tego samego urzdzenia slave
   protok: <Start><Slv1A><Read><Ack><D1><Ack>...<Dnum1><Nack>
             <Start><Slv2A><Write><Ack><D2><Ack>...<Dnum2><Ack><Stop> */
void I2C_ReadRepWrite(I2C_MESSAGE *msg1, I2C_MESSAGE *msg2)
{
	iicTfr.nrMessages = 2;
	iicTfr.p_message = p_iicMsg;
	p_iicMsg[0] = msg1;
	p_iicMsg[1] = msg2;
	
	StartTransfer();
}

