;**************************************************************************;* AVR ISP (Auto inkrementacja adresu, rs232  19200,n,8,1)
;* Wersja	: 2.0
;* Mikrokontroler	: AT90S1200
;* Autor	: Atmel Norway, Nota aplikacyjna AVR910
;**************************************************************************

;doczenie zbioru definicji rejestrw mikrokontrolera
.include "1200def.inc"

;definicje kodw programowanych ukadw
.equ	S1200C	= 0x12
.equ	S1200D	= 0x13
.equ	S2313A	= 0x20
.equ	S4414A	= 0x28
.equ	S8515A	= 0x38
.equ	S8252		= 0x86
.equ	S2323A	= 0x48
.equ	S01838C	= 0x40
.equ	S01838D	= 0x41

;wersja programu i programatora
.equ	SW_MAJOR = 2
.equ	SW_MINOR = 0
.equ	HW_MAJOR = 1
.equ	HW_MINOR = 0

;definicje staych zwizane z programowym interfejsem SPI
;numery bitw wyprowadze interfejsu
.equ	RESET	= 4
.equ	MOSI	= 6
.equ	MISO	= 5
.equ	SCK	= 7

;definicje linii, do ktrych podczone s diody LED
.equ	LEDOK		= 4
.equ	LEDFAULT	= 5
.equ	LEDWRK	= 6

;definicje staych zwizane z programowym interfejsem RS232 (UART)
;- szybko transmisji UART
;.equ	N = 95			;115200 BAUD gdy R=1 i XTAL=11.059MHz
;.equ	N = 31			;57600 BAUD gdy R=2 i XTAL=11.059MHz
;.equ	N = 43			;38400 BAUD gdy R=2 i XTAL=11.059MHz
.equ	N = 33			;19200 BAUD gdy R=2 i XTAL=4.00MHz
;.equ	N = 102			;38400 BAUD gdy R=1 i XTAL=4.00MHz
.equ	R = 2

;- numery wyprowadze UART
.equ	TXPIN = 1
.equ	RXPIN = 0			;pin RxD musi by wejciem przerwania

;pozycje bitw w rejestrze statusu UART
.equ	TXC = 0			;nadawanie
.equ	RXC = 1			;odbir


;definicje makr
.macro	OK_on				;zaczenie diody OK
	cbi	portd,LEDOK
.endm

.macro	FAULT_on			;zaczenie diody BD
	cbi	portd,LEDFAULT
.endm

.macro	WORKING_on			;zaczenie diody ZAPIS/ODCZYT
	cbi	portd,LEDWRK
.endm

.macro	OK_off			;wyczenie diody OK
	sbi	portd,LEDOK
.endm

.macro	FAULT_off			;wyczenie diody BD
	sbi	portd,LEDFAULT
.endm

.macro	WORKING_off			;wyczenie diody ZAPIS/ODCZYT
	sbi	portd,LEDWRK
.endm

.macro	set_reset			;sygna reset aktywny
	sbi	portb,RESET
.endm

.macro	clr_reset			;usunicie sygnau reset
	cbi	portb,RESET
.endm

.macro	ddrd_init			;inicjowanie portu D
	nop
	sbi	ddrd,3
.endm

.macro	ddrb_init			;inicjowanie portu B
	ldi	temp1,0xdf
	out	ddrb,temp1			;PB5 jest wejciem, reszta to wyjcia
.endm

.macro	ddrb_release
	ldi	temp1,(1<<PB4)
	out	ddrb,temp1			;PB4 (reset) wyjcie, reszta to wejcia
.endm

.macro	pulse_sck			;impuls na linii SCK interfejsu SPI
	sbi	portb,SCK
	ldi	temp2,6
m0:
	dec	temp2
	brne	m0
	cbi	portb,SCK
	ldi	temp2,1			; 3
m1:
	dec	temp2
	brne	m1
.endm

;definicje zmiennych
.def	device	= r16			;kod programowanego ukadu
.def	temp1		= r17
.def	temp2		= r18
.def	s_data	= r19			;bufor danych spi
.def	u_data	= r20			;bufor danych UART
.def	addrl		= r21			;mniej znaczcy bajt adresu
.def	addrh		= r22			;bardziej znaczcy bajt adresu
.def	bit_cnt	= r23			;licznik bitw dla UART
.def	u_stat	= r24			;bajt statusu UART
.def	cmd		= r25			;komenda programowania szeregowego 
.def	count		= r26			;zmienna limitu czasu operacji
.def	param1	= r27
.def	cmd1		= r28
.def	cmd2		= r29
.def	cmd3		= r30

;wektory przerwa
.CSEG
	rjmp	RESET				;Reset
	reti					;nie uywane INT0
	rjmp	TIM0_OVF			;Timer 0
	reti					;nie uywane przerwanie od komparatora

;TIM0_OVF  procedura obsugi programowego interfejsu UART
TIM0_OVF:
	in	r0,SREG			;zapamitanie SREG
	ldi	temp1,(256-N+8)
	out	TCNT0,temp1			;zerowanie T/C0 dla 1-bitowej dugoci
	inc	bit_cnt			;zwikszenie licznika bitw o 1
	sbrs	u_stat,TXC			;zerowanie flagi,jeli koniec transmisji
	rjmp	transmit			;skok do transmit

to_0:
	sec					;ustawienie flagi carry
	sbis	PIND,RXPIN			;jeli wyprowadzenie RxD = 0
	clc					;	to kasowanie carry
	ror	u_data			;przesunici w prawo, wprowadzenie carry
						;do odbieranego sowa
	cpi	bit_cnt,8			;czy licznik bitw = 8?
	brne	to_1				;nie, dalej obsuguj odbir z UART
	clr	temp1				;tak, wycz przerwanie T/C0
	out	TIMSK,temp1
	sbr	u_stat,1<<RXC		;ustaw flag zakoczenia odbioru
to_1:
	out	SREG,r0	        	;odtwrz SREG
	reti					;wyjcie

transmit:
	cpi	bit_cnt,1			;jeli bit_cnt = 1 wylij bit start
	brne	to_2
	cbi	PORTD,TXPIN
	rjmp	to_1
to_2:
	cpi	bit_cnt,10			;jeli bit_cnt = 10 wylij bit stop
	brne	to_3
	sbi	PORTD,TXPIN
	clr	temp1				;wycz przerwanie T/C0
	out	TIMSK,temp1
	sbr	u_stat,1<<TXC		;ustaw flag zakoczenia transmisji
	rjmp	to_1
to_3:
	sbrc	u_data,0			;jeli najstarszy bit jest 1 
	sbi	PORTD,TXPIN			;ustaw wyjcie TXD
	sbrs	u_data,0			;jeli najstarszy bit jest 0
	cbi	PORTD,TXPIN			;zeruj wyjcie TXD
	lsr	u_data			;przesu w lewo zmienn u_data
	rjmp	to_1

;U_INIT inicjalizacja programowego interfejsu UART
u_init:
	ldi	u_stat,1<<TXC		;ustawienie flagi TXC
	ldi	temp1,R			;ustawienie prdkoci tranmisji
	out	TCCR0,temp1
	sbi	DDRD,TXPIN			;inicjalizacja wyj UART
	cbi	DDRD,RXPIN
	ret

;PUTC wysanie znaku przez UART
putc:
	clr	u_stat			;kasowanie flag statusu UART
	clr	bit_cnt			;kasowanie licznika bitw
	ldi	temp1,1<<TOV0		;zaczenie przerwania T/C0
	out	TIMSK,temp1
putc0:
	sbrs	u_stat,TXC			;oczekiwanie na bit sygnalizujcy
	rjmp	putc0				;zakoczenie wysyania sowa
	ret


;GETC oczekiwanie na bit startu i odebranie znaku przez UART
getc:
	sbis	PIND,RXPIN
	rjmp	getc
getc0:
	sbic	PIND,RXPIN
	rjmp	getc0
	ldi	temp1,(256-(N+N/2)+8+12)
	out	TCNT0,temp1 		;ustawienie T/C0 na 1.5 bit
	ldi	temp1,1<<TOIE0
	out	TIFR,temp1			;kasowanie flagi przepenienia T/C0
	out	TIMSK,temp1			;zaczenie przerwania T/C0
	clr	bit_cnt			;kasowanie licznika bitw
getc1:
	sbrs	u_stat,RXC			;oczekiwanie na zakoczenie odbioru
	rjmp	getc1				;znaku przez UART
	cbr	u_stat,1<<RXC		;kasowanie flagi RXC
	ret

;DELAY krtkie opnienie, okoo 1 msec
delay:
	ldi	temp2,0xff
dl:
	dec	temp2
	brne	dl
	dec	temp1
	brne	delay
	ret

;WRSER wysanie bajtu przez interfejs SPI
wrser:
	ldi	temp1,8
wrs0:	rol	s_data
	brcc	wrs1
	sbi	portb,MOSI
	rjmp	wrs2
wrs1:
	cbi	portb,MOSI
wrs2:
	pulse_sck
	dec	temp1
	brne	wrs0
	ret

;RDSER odbir i odczyt bajtu z SPI
rdser:
	ldi	temp1,8
	ldi	s_data,0
rd0:
	lsl	s_data
	sbic	pinb,MISO
	ori	s_data,1
	pulse_sck
	dec	temp1
	brne	rd0
	ret

;SPIINIT inicjalizacja interfejsu SPI w programowanym ukadzie
spiinit:
ddrd_init				;ustawienie portu D
	ddrb_init				;ustawienie portu B
	cbi	portb,SCK			;kasowanie linii SCK
	cpi	device,S8252		;czy programowany ukad to AT89S8252?
	breq	s89				;nie, aktywnym stanem reset jest 0
	set_reset				;RESET = 1
	ldi	temp1,0xff			;256 x delay
	rcall	delay
	clr_reset				;RESET = 0
	rjmp	s0
s89:						;dla AT89 aktywny reset to 1
	clr_reset				;RESET = 0
	ldi	temp1,0xff			;256 x delay
	rcall	delay
	set_reset				;RESET = 1
s0:
	ldi	temp1,0xff			;256 x delay
	rcall	delay

	ldi	s_data,0xac			;wysanie bajtu nr 1 = ACh przez SPI
	rcall	wrser
	ldi	s_data,0x53			;bajt nr 2 = 53h
	rcall	wrser
;synchronizacja interfejsu SPI, uwzgldnienie bdu firmware dla niektrych
;typw mikrokontrolerw ATMEL 
	cpi	device,0x20			;jeli identyfikator
	brlo	s2				;ukadu >= 20h i id. <=0x7F
	tst	device
	brmi	s2
s0b:
	ldi	count,32
s1:
	rcall	rdser
	cpi	s_data,0x53
	breq	s3
	ldi	s_data,0x00			;bajt 4 = 0h
	rcall	wrser
	pulse_sck				;impuls na linii SCK
	ldi	s_data,0xac			;zapis bajtu nr 1 = ACh
	rcall	wrser
	ldi	s_data,0x53			;bajt 2 = 53h
	rcall	wrser
	dec	count				;dopki licznik wikszy od 0
	brne	s1
	rjmp	s3
s2:
	ldi	s_data,0x00			;bajt 3 = 0h
	rcall	wrser
s3:
	cpi	device,S8252		;jeli nie programujemy AT89S8252
	breq	s4				;to zapis bajtu 4 (0h)
	ldi	s_data,0x00			;inaczej wyjcie
	rcall	wrser
s4:
	ldi	temp1,0x10
	rcall	delay
	ret

;SHOW_ID wysanie identyfikatora AVR ISP przez UART
show_id:
ldi	u_data,0x41			;wysanie 'A'
	rcall	putc
	ldi	u_data,0x56			; 'V'
	rcall	putc
	ldi	u_data,0x52			; 'R'
	rcall	putc
	ldi	u_data,0x20			; ' '
	rcall	putc
	ldi	u_data,0x49			; 'I'
	rcall	putc
	ldi	u_data,0x53			; 'S'
	rcall	putc
	ldi	u_data,0x50			; 'P'
	rcall	putc
	ret


;**************************************************************************
;inicjalizacja programatora po reset
;pocztek programu gwnego
;**************************************************************************
RESET:
	clr	temp1
	out	GIMSK,temp1			;wyczenie przerwa zewntrznych
	ser	temp1
	out	PORTD,temp1
	set_reset				;ustawienie sygnau RESET
	out	PORTB,temp1
	ddrb_release
	rcall	u_init			;inicjalizacja UART
	LEDOK_on				;zaczenie zielonej diody LED, reszta
	LEDFAULT_off			;ma by zgaszona
	LEDWRK_off
	sei					;zaczenie przerwa

;ptla gwna  oczekiwanie na komendy z komputera PC
waitcmd:
rcall	getc				;ptla wykonywana dopki odebrane znaki
	cpi	u_data,0x1b			;s rwne kodowi ESC (1Bh)
	breq	waitcmd

;realizacja komendy Device Type programatora (typ programowanego ukadu)
	cpi	u_data,0x54     		;'T' = Device Type
	brne	w0
	rcall	getc
	mov	device,u_data
	rjmp	put_ret

;Return Software Identifier (zwr wersj oprogramowania)
w0:	cpi	u_data,0x53			;'S' = Return software identifier
	brne	w1
	rcall	show_id			;wysanie znakw AVR ISP
	rjmp	waitcmd

;Return Software Version (zwr wersj programu programatora)
w1:	cpi	u_data,0x56			;'V' = Return software version
	brne	w2
	ldi	u_data, SW_MAJOR
	rcall	putc
	ldi	u_data,SW_MINOR
	rcall	putc
	rjmp	waitcmd

;Return Hardware Version (zwr wersj programatora)
w2:	cpi	u_data,0x76			;'v' = Return hardware version
	brne	w3
	ldi	u_data,HW_MAJOR
	rcall	putc
	ldi	u_data,HW_MINOR
	rcall	putc
	rjmp	waitcmd

;Show Supported Devices (wylij list obsugiwanych ukadw)
w3:	cpi	u_data,0x74			;'t' = Show supported devices
	brne	w4
	ldi	u_data,S1200C		;AT90S1200C
	rcall	putc
	ldi	u_data,S1200D		;AT90S1200D
	rcall	putc
	ldi	u_data,S2313A		;AT90S2313A
	rcall	putc
	ldi	u_data,S4414A		;AT90S4414A
	rcall	putc
	ldi	u_data,S8515A		;AT90S8515A
	rcall	putc
	ldi	u_data,S2323A		;AT90S2323A
	rcall	putc
	ldi	u_data,S8252		;AT89S8252
	rcall	putc
ldi	u_data,S01838C		;ATMega 103C
rcall	putc
ldi	u_data,S01838D		;ATMega 103D
rcall	putc
	ldi	u_data,0x00			;znak koca listy
	rcall	putc
	rjmp	waitcmd

;Return Programmer Type (wylij typ programatora)
w4:	cpi	u_data,0x70			;'p' = Return programmer type
	brne	w5
	ldi	u_data,0x53     		;'S' = programator szeregowy
	rcall	putc
	rjmp	waitcmd

;Set LED (zaczenie diody LED)
w5:	cpi	u_data,0x78			;'x' = Set LED
	brne	w6
	rcall	getc				;odbierz drugi bajt rozkazu
	cpi	u_data,0x00			;warto 0 oznacza zaczenie diody BD
	brne	w51
	LEDFAULT_on
	rjmp	put_ret
w51:
	cpi	u_data,0x01			;warto 1 oznacza zaczenie diody OK
	brne	w52
	LEDOK_on
	rjmp	put_ret
w52:
	cpi	u_data,0x02			;warto 2 oznacza zaczenie diody
	brne	w6				;ZAPIS/ODCZYT
	LEDWRK_on
	rjmp	put_ret

;Clear LED (wyczenie diody LED)
w6:	cpi	u_data,0x79     		;'y' = Clear LED (ignored)
	brne	w7
	rcall	getc				;odbierz drugi bajt rozkazu
	cpi	u_data,0x00			;warto 0 oznacza wyczenie diody BD
	brne	w61
	LEDFAULT_off
	rjmp	put_ret
w61:
	cpi	u_data,0x01			;warto 1 oznacza wyczenie diody OK
	brne	w62
	LEDOK_off
	rjmp	put_ret
w62:
	cpi	u_data,0x02			;warto 2 oznacza wyczenie diody
	brne	w7				;ZAPIS/ODCZYT
	LEDWRK_off
	rjmp	put_ret


;kod ukadu musi zosta wybrany zanim moliwe bdzie wykonanie 
;ktrejkolwiek z poniszych komend
w7:
	cpi	device,S1200C
	breq	w72
	cpi	device,S1200D
	breq	w72
	cpi	device,S8515A
	breq	w72
	cpi	device,S4414A
	breq	w72
	cpi	device,S2313A
	breq	w72
	cpi	device,S8252
	breq	w72
	cpi	device,S01838C
	breq	w72
	cpi	device,S01838D
	breq	w72
	cpi	device,S2323A
	breq	w72
	rjmp	put_err			;jeli kod ukadu nie by ustawiony, to
						;komunikat o bdzie!

;Enter Programming Mode (zaczenie trybu programowania)
w72:	cpi	u_data,0x50			;'P' = Enter programming mode
	brne	w8
	rcall	spiinit			;inicjalizacja interfejsu SPI
	rjmp	put_ret

;Write Program Memory, High Byte (zapis starszego bajtu do pamici)
w8:	cpi	u_data,0x43			;'C' = Write program memory, high byte
	brne	w9
	rcall	getc
	cpi	device,S8252		;jeli wybrano AT89, to sowo ma by
	brne	w81				;jednobajtowe (inaczej ni w AVR)
	rjmp	put_err

w81:	ldi	s_data,0x48			;bajt 1 = 0x48
	rcall	wrser
	mov	s_data,addrh		;bajt 2 = addrh
	rcall	wrser
	mov	s_data,addrl		;bajt 3 = addrl
	rcall	wrser
	cpi	device,S01838C		;negowanie bajtu dla data 01838 rev.C
	brne	w81b
	com	u_data
w81b:
	mov	s_data,u_data		;bajt 4 = dane
	cpi	device,S01838C		;bez opnienia da S01838
breq	w82				;uywany jest tryb zapisu strony
	cpi	device,S01838D
	breq	w82

	ldi	temp1,0x20			;opnienie 24585 cykli
	rcall	delayw82:
	ldi	temp1,0x01			;zwikszenie adresu
	clr	temp2
	add	addrl,temp1
	adc	addrh,temp2
	rjmp	put_ret

;Write Program Memory, Low Byte (zapis pamici programu, modszy bajt)
w9:	cpi	u_data,0x63     		;'c' = Write program memory, low byte
	brne	w12
	rcall	getc

	cpi	device,S8252		;czy wybrano AT89S8252?
	breq	w989				;tak, skocz do etykiety w989
	ldi	s_data,0x40			;nie, wylij przez SPI 40h
	rcall	wrser
	mov	s_data,addrh		;i starszy bajt adresu
	rjmp	w91
w989:
mov	s_data,addrh		;dla AT89 specjalna konstrukcja sowa
rcall	shift_s_data3		;sowo = (addrh << 3) or 0x02
	ori	s_data,0x02
w91:
	rcall	wrser				;zapis bajtu nr 2
	mov	s_data,addrl		;zapis bajtu nr 3
	rcall	wrser
	cpi	device,S01838C		;zanegowanie bajtu dla 01838 rev.C
	brne	w91b
	com	u_data
w91b:
	mov	s_data,u_data		;zapis bajtu nr 4
	rcall	wrser
	cpi	device,S01838C		;bez opnienia dla ATMega
	breq	w92
	cpi	device,S01838D
	breq	w92
	ldi	temp1,0x20			;opnienie = 24585 cykli
	rcall	delay
w92:
	rjmp	put_ret

;Read Program Memory (odczyt pamici programu)
w12:	cpi	u_data,0x52			;'R' = Read program memory
	brne	w10
	cpi	device,S8252		;jeli wybrano AT89S8252 skocz do w1289
	breq	w1289
	ldi	s_data,0x28			;wylij przez SPI 28h
	rcall	wrser
	mov	s_data,addrh		;s_data = addrh
	rjmp	w121
w1289:
	mov	s_data,addrh		;specjalna konstrukcja sowa dla AT89
rcall	shift_s_data3 		;sowo = (addrh << 3) or 0x01
	ori	s_data,0x01
w121:
	rcall	wrser				;zapis sowa (bajt nr 2)
	mov	s_data,addrl		;bajt 3 = addrl
	rcall	wrser
	rcall	rdser				;bajt 4
	mov	u_data,s_data
	rcall	putc
	cpi	device,S8252		;jeli wybrano AT89S8252, to koniec
	brne	w122				;jeli inny, zapis addrh
	rjmp	waitcmd
w122:
	ldi	s_data,0x20			;bajt 1 = 20h
	rcall	wrser
	mov	s_data,addrh		;bajt 2 = addrh (starsza cz adresu)
	rcall	wrser
	mov	s_data,addrl		;bajt 3 = addrl
	rcall	wrser
	rcall	rdser				;bajt 4 = dane
	mov	u_data,s_data
	rcall	putc

	ldi	temp1,0x01			;wykonanie funkcji Autoincrement Address
	clr	temp2				;automatyczne zwikszenie adresu o 1
	add	addrl,temp1
	adc	addrh,temp2
	rjmp	waitcmd

;przesunicie odebranego przez uart bajtu o 3 bity w lewo
shift_s_data3:
	lsl	s_data
	lsl	s_data
	lsl	s_data
	ret

;Load Address (ustalenie adresu)
w10:	cpi	u_data,0x41			;'A' = Load address
	brne	w11
	rcall	getc				;addrh
	mov	addrh,u_data
	rcall	getc				;addrl
	mov	addrl,u_data
	rjmp	put_ret

;Write Data Memory (zapis pamici danych - eeprom)
w11:	cpi	u_data,0x44			;'D' = Write data memory
	brne	w13
	rcall	getc
	cpi	device,S8252		;ponownie inny algorytm dla AT89
	breq	w1189

	ldi	s_data,0xc0
	rcall	wrser
	mov	s_data,addrh
	rjmp	w111

w1189:
	mov	s_data,addrh
	rcall	shift_s_data3
	ori	s_data,0x06

w111:
rcall	wrser
	mov	s_data,addrl
	rcall	wrser
	mov	s_data,u_data
	rcall	wrser
	ldi	temp1,0x20
	rcall	delay
	rjmp	put_ret

;Read Data Memory (odczyt pamici danych)
w13:	cpi	u_data,0x64			;'d' = Read data memory
	brne	w14
	cpi	device,S8252		;ponownie inaczej dla AT89
	breq	w1389
	ldi	s_data,0xA0
	rcall	wrser
	mov	s_data,addrh
	rjmp	w131
w1389:
	mov	s_data,addrh
	rcall	shift_s_data3
	ori	s_data,0x05
w131:
	rcall	wrser
	mov	s_data,addrl
	rcall	wrser
	rcall	rdser
	mov	u_data,s_data
	rcall	putc
	rjmp	waitcmd

;Leave Programming Mode (wyjcie z trybu programowania)
w14:	cpi	u_data,0x4c			;'L' = Leave programming mode
	brne	w15
	ddrb_release
	cpi	device,S8252		;dla AT89 odwrotna faza sygnau reset
	breq	w141
	set_reset				;RESET = 1
	rjmp    put_ret
w141:	clr_reset				;RESET = 0
	rjmp	put_ret

;Chip Erase (kasowanie pamici programu i danych)
w15:	cpi	u_data,0x65     		;'e' = Chip erase
brne	w16
	ldi	s_data,0xac
	rcall	wrser
	cpi	device,S8252
	breq	w1589
	ldi	s_data,0x80
	rcall	wrser
w1589:
	ldi	s_data,0x04
	rcall	wrser
	ldi	s_data,0x00
	rcall	wrser
	ldi	temp1,0x30
	rcall	delay
	rjmp	put_ret

;Write Lock Bits (zapis bitw blokujcych)
w16:	cpi	u_data,0x6c			;'l' = Write lock bits
	brne	w17
	rcall	getc
	ldi	s_data,0xac
	rcall	wrser
	mov	s_data,u_data
	cpi	device,S8252
	breq	w1689
	andi	s_data,0x06
	ori	s_data,0xe0
	rcall	wrser
	ldi	s_data,0x00
	rcall	wrser
	rjmp	w162
w1689:
	andi	s_data,0xe0
	ori	s_data,0x07
	rcall	wrser
w162:
	ldi	s_data,0x00
	rcall	wrser
	ldi	temp1,0x30
	rcall	delay
	rjmp	put_ret

;Read Signature Bytes (odczyt bajtu sygnatury)
w17:	cpi	u_data,0x73			;'s' = Read signature bytes
	brne	w18
	cpi	device,S8252
	brne	w17begin
	rjmp	put_err
w17begin:
	ldi	param1,0x02
	rcall	w17call
	ldi	param1,0x01
	rcall	w17call
	ldi	param1,0x00
	rcall	w17call
	rjmp	waitcmd
w17call:
ldi	s_data,0x30
	rcall	wrser
	ldi	s_data,0x00
	rcall	wrser
	mov	s_data,param1
	rcall	wrser
	rcall	rdser
	mov	u_data,s_data
	rcall	putc
	ret

;Write Program Memory Page (zapis strony pamici programu  ATMega)
w18:	cpi	u_data,0x6D			;'m' = Write Program Memory Page
	brne	w19
	ldi	s_data,0x4c			;bajt 1 = 4CH
	rcall	wrser
	mov	s_data,addrh		;bajt 2 = addrh
	rcall	wrser
	mov	s_data,addrl		;bajt 3 = addrl
	rcall	wrser
	ldi	s_data,0x00			;bajt 4 = 0
	rcall	wrser
	ldi	temp1,0xff			;opnienie = 24585 cykli
	rcall	delay
	rjmp	put_ret

;Universal Command (komenda uniwerslana)
w19:	cpi	u_data,0x3A			;':' = Universal Command
	brne	w99
	rcall	getc
	mov	cmd1,u_data
	rcall	getc
	mov	cmd2,u_data
	rcall	getc
	mov	cmd3,u_data
	rcall	universal
	ldi	temp1,0xff			;opnienie = 24585 cykli
	rcall	delay
	rjmp	put_ret

universal:
	mov	s_data,cmd1
	rcall	wrser
	mov	s_data,cmd2
	rcall	wrser
	mov	s_data,cmd3
	rcall	wrser
	rcall	rdser
	mov	u_data,s_data
	rcall	putc
	ret

w99:

;Command Error (bd programatora, brak moliwoci wykonania komendy)
put_err:
ldi	u_data,0x3f			;wylij przez UART kod znaku ?
	rcall	putc
	rjmp	waitcmd

;Reply Command (potwierdzenie odbioru komendy)
put_ret:
ldi	u_data,0x0d			;wysanie kodu CR
	rcall	putc
	rjmp	waitcmd

;End of File
END
